@@ -1,16 +1,15 @@
 /*
- * compat-mit.c
- *
  * Kerberos compatibility functions for MIT Kerberos.
+ *
+ * Copyright 2005, 2006, 2007 Russ Allbery <rra@debian.org>
+ * Copyright 2005 Andres Salomon <dilinger@debian.org>
+ * Copyright 1999, 2000 Frank Cusack <fcusack@fcusack.com>
+ *
+ * See LICENSE for licensing terms.
  */
 
 #include "config.h"
 
-#ifdef HAVE_ET_COM_ERR_H
-# include <et/com_err.h>
-#else
-# include <com_err.h>
-#endif
 #include <errno.h>
 #include <krb5.h>
 #include <stdlib.h>
@@ -25,40 +24,11 @@ pamk5_compat_free_data_contents(krb5_context c, krb5_data *data)
 }
 
 
-#ifdef HAVE_KRB5_GET_ERROR_MESSAGE
-const char *
-pamk5_compat_get_error(krb5_context c, krb5_error_code code)
-{
-    const char *msg;
-
-    msg = krb5_get_error_message(c, code);
-    if (msg == NULL)
-        return "unknown error";
-    else
-        return msg;
-}
-#else /* !HAVE_KRB5_GET_ERROR_MESSAGE */
-const char *
-pamk5_compat_get_error(krb5_context c, krb5_error_code code)
-{
-    return error_message(code);
-}
-#endif
-
-
-#ifdef HAVE_KRB5_FREE_ERROR_MESSAGE
 void
-pamk5_compat_free_error(krb5_context c, const char *msg)
+pamk5_compat_free_keytab_contents(krb5_context c, krb5_keytab_entry *entry)
 {
-    krb5_free_error_message(c, msg);
+    krb5_free_keytab_entry_contents(c, entry);
 }
-#else /* !HAVE_KRB5_FREE_ERROR_MESSAGE */
-void
-pamk5_compat_free_error(krb5_context c, const char *msg)
-{
-    return;
-}
-#endif /* !HAVE_KRB5_FREE_ERROR_MESSAGE */
 
 
 krb5_error_code
