@@ -1,18 +1,27 @@
 /*
- * prompting.c
- *
  * Prompt users for information.
  *
  * Handles all interaction with the PAM conversation, either directly or
  * indirectly through the Kerberos libraries.
+ *
+ * Copyright 2005, 2006, 2007 Russ Allbery <rra@debian.org>
+ * Copyright 2005 Andres Salomon <dilinger@debian.org>
+ * Copyright 1999, 2000 Frank Cusack <fcusack@fcusack.com>
+ *
+ * See LICENSE for licensing terms.
  */
 
 #include "config.h"
 
 #include <errno.h>
 #include <krb5.h>
-#include <security/pam_appl.h>
-#include <security/pam_modules.h>
+#ifdef HAVE_SECURITY_PAM_APPL_H
+# include <security/pam_appl.h>
+# include <security/pam_modules.h>
+#elif HAVE_PAM_PAM_APPL_H
+# include <pam/pam_appl.h>
+# include <pam/pam_modules.h>
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -121,8 +130,8 @@ pamk5_conv(struct pam_args *args, const char *message, int type,
 {
     int pamret;
     struct pam_message msg;
-    const struct pam_message *pmsg;
-    struct pam_response	*resp = NULL;
+    PAM_CONST struct pam_message *pmsg;
+    struct pam_response *resp = NULL;
     struct pam_conv *conv;
     int want_reply;
 
@@ -133,7 +142,7 @@ pamk5_conv(struct pam_args *args, const char *message, int type,
 	return pamret;
     pmsg = &msg;
     msg.msg_style = type;
-    msg.msg = message;
+    msg.msg = (PAM_CONST char *) message;
     pamret = conv->conv(1, &pmsg, &resp, conv->appdata_ptr);
     if (pamret != PAM_SUCCESS)
 	return pamret;
@@ -195,7 +204,7 @@ pamk5_conv(struct pam_args *args, const char *message, int type,
  * into the krb5_prompt structs to return to the Kerberos library.
  */
 krb5_error_code
-pamk5_prompter_krb5(krb5_context context, void *data, const char *name,
+pamk5_prompter_krb5(krb5_context context UNUSED, void *data, const char *name,
                     const char *banner, int num_prompts, krb5_prompt *prompts)
 {
     struct pam_args *args = data;
@@ -279,7 +288,7 @@ pamk5_prompter_krb5(krb5_context context, void *data, const char *name,
     }
 
     /* Call into the application conversation function. */
-    pamret = conv->conv(pam_prompts, (const struct pam_message **) msg,
+    pamret = conv->conv(pam_prompts, (PAM_CONST struct pam_message **) msg,
                         &resp, conv->appdata_ptr);
     if (pamret != 0) 
         goto cleanup;
